#include "Texture.h"

#include <SOIL\SOIL.h>

using namespace Inbetween;
/*
int height[];
int width[];
unsigned char* image[];
*/
Texture::Texture()
{
}

Texture::Texture(const char* path)
{
	loadTexture(path);
}

void Inbetween::Texture::loadTexture(const char* path)
{
	glGenTextures(1, &handle);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, handle);

	int width2, height2;
	//auto data = LoadTexture(path, width, height); //width and height are output params!!!
	unsigned char* image = SOIL_load_image(path, &width2, &height2, 0, SOIL_LOAD_RGB);


	glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGB,
		width2,
		height2,
		0,
		GL_RGB,
		GL_UNSIGNED_BYTE,
		image);
	glGenerateMipmap(GL_TEXTURE_2D);



	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

GLuint Texture::loadCubemap(std::vector<const GLchar *> faces) {

	GLuint textureID;
	glGenTextures(1, &handle);

	int width, height;
	unsigned char* image;

	glBindTexture(GL_TEXTURE_CUBE_MAP, handle);
	for (GLuint i = 0; i < faces.size(); i++)
	{
		image = SOIL_load_image(faces[i], &width, &height, 0, SOIL_LOAD_RGB);
		glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
		SOIL_free_image_data(image);
	}
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glBindTexture(GL_TEXTURE_CUBE_MAP, 0);

	return handle;
}

Texture::~Texture()
{
	glDeleteTextures(1, &handle);
}

void Texture::bind(int unit)
{
	glActiveTexture(GL_TEXTURE0 + unit);
	glBindTexture(GL_TEXTURE_2D, handle);
	mipMaping(this->mipmaping);
}

GLuint Texture::textureSampling() {
	switch (this->texturesampling)
	{
	case 0:
		return GL_NEAREST;
	case 1:
		return GL_LINEAR;
	}
	return -1;
}

void Texture::mipMaping(int mipmaping) {
	switch (mipmaping) {
	case 0:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, textureSampling());
		break;
	case 1:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
		break;
	case 2:
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	}
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, textureSampling());

}
