#version 330

struct Material {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;    
    float shininess;
}; 

struct Light {
    vec3 direction;

    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};
  
in vec3 fragNormal;
in vec2 fragmentUV;
in vec3 fragPos;  
in vec4 FragPosLightSpace;

layout(location=0) out vec4 color;
layout (location = 1) out vec4 BrightColor;

uniform sampler2D color_texture;
uniform sampler2D shadowMap;

uniform vec3 viewPos;
//uniform vec3 lightPos;

uniform Material material;
uniform Light light;

uniform bool celshading;

const float levels = 5.0;
const float levelspec = 3.0;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // Check whether current frag pos is in shadow
	//float bias = max(0.05 * (1.0 - dot(fragNormal, normalize(-light.direction))), 0.005);  
	float bias = 0.0035;
	 float shadow = 0.0;
	 if(projCoords.z > 1.0){
        shadow = 0.0;
		}else{
		vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
        for(int x = -1; x <= 1; ++x){
              for(int y = -1; y <= 1; ++y) {
                  float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
                  shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
              }    
        }
        shadow /= 9.0;
	}

    return shadow;
}

void main()
{
	
	vec3 textureColor = texture(color_texture, fragmentUV).rgb;

	 // Ambient
    vec3 ambient = (light.ambient * material.ambient);
  	
    // Diffuse 
    vec3 norm = normalize(fragNormal);
	//vec3 lightDir = normalize(lightPos - fragPos);
    vec3 lightDir = normalize(-light.direction);
    float diff = max(dot(norm, lightDir), 0.0);

    //clamp values to get cel shading look
	if(celshading){
	float level = floor(diff * levels);
	diff = level / levels;
	}
    vec3 diffuse = light.diffuse * (diff * material.diffuse);
	    
    // Specular
    vec3 viewDir = normalize(viewPos - fragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);

	//clamp values to get cel shading look
	if(celshading){
	float level2 = floor(spec * levelspec);
	spec = level2 / levelspec;
	}

    vec3 specular = light.specular * (spec * material.specular);  

	float shadow = ShadowCalculation(FragPosLightSpace);       
    vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * textureColor;   

   // vec3 result = (ambient + diffuse + specular ) * textureColor;

   float brightness = dot(lighting, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 0.35)
	BrightColor = vec4(lighting, 1.0);

    color = vec4(lighting, 1.0f);

	/*
	vec3 textureColor = texture(color_texture, fragmentUV).rgb;
    vec3 normal = normalize(fragNormal);
    vec3 lightColor = vec3(1.0);
    // ambient
    vec3 ambient = 0.15 * textureColor;
    // diffuse
    vec3 lightDir = normalize(lightPos - fragPos);
    float diff = max(dot(lightDir, normal), 0.0);
    vec3 diffuse = diff * lightColor;
    // specular
    vec3 viewDir = normalize(viewPos - fragPos);
    float spec = 0.0;
    vec3 halfwayDir = normalize(lightDir + viewDir);  
    spec = pow(max(dot(normal, halfwayDir), 0.0), 32.0);
    vec3 specular = spec * lightColor;    
    // calculate shadow
    float shadow = ShadowCalculation(FragPosLightSpace);       
    vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * textureColor;	
    
	
	float brightness = dot(lighting, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 0.35)
	BrightColor = vec4(lighting, 1.0);

    color = vec4(lighting, 1.0); 
	*/

}